@@ -2,7 +2,7 @@ package RedisDB;
 
 use warnings;
 use strict;
-our $VERSION = "0.27";
+our $VERSION = "0.29";
 $VERSION = eval $VERSION;
 
 use RedisDB::Error;
@@ -28,14 +28,15 @@ RedisDB - Perl extension to access redis database
 
 =head1 DESCRIPTION
 
-This module provides interface to access redis database. It transparently
-handles disconnects and forks. It supports pipelining mode.
+This module provides interface to access redis key-value store. It
+transparently handles disconnects and forks. It supports transactions,
+pipelining, and subscription mode.
 
 =head1 METHODS
 
 =head2 $class->new(%options)
 
-Creates new RedisDB object. The following options are allowed:
+Creates the new RedisDB object. The following options are accepted:
 
 =over 4
 
@@ -50,18 +51,19 @@ port to connect. Default: 6379
 =item path
 
 you can connect to redis using UNIX socket. In this case instead of
-I<host> and I<port> you should specify I<path>.
+L</host> and L</port> you should specify I<path>.
 
 =item timeout
 
-IO timeout. With this option set, if IO operation will take more than specified
-number of seconds module will croak. Note, that some OSes don't support SO_RCVTIMEO,
-and SO_SNDTIMEO socket options, in this case timeout will not work.
+IO timeout. With this option set, if IO operation has taken more than specified
+number of seconds, module will croak. Note, that some OSes do not support
+SO_RCVTIMEO, and SO_SNDTIMEO socket options, in this case timeout will not
+work.
 
 =item lazy
 
 by default I<new> establishes connection to the server. If this parameter is
-set, then connection will be established when you will send command to the
+set, then connection will be established when you will send a command to the
 server.
 
 =back
@@ -86,18 +88,19 @@ sub new {
 
 =head2 $self->execute($command, @arguments)
 
-send command to the server and return server reply. It throws exception if
-server returns error. It may be more convenient to use instead of this method
-wrapper named after the redis command. E.g.:
+send a command to the server and return the result. It will throw the exception
+if the server returns an error. It may be more convenient to use instead of
+this method wrapper named after the corresponding redis command. E.g.:
 
     $redis->execute('set', key => 'value');
     # is the same as
     $redis->set(key => 'value');
 
-See "SUPPORTED REDIS COMMANDS" section for the full list of defined aliases.
+See L</"WRAPPER METHODS"> section for the full list of defined aliases.
 
-Note, that you can't use I<execute> if you have sent some commands in pipelining
-mode and haven't yet got all replies.
+Note, that you can't use I<execute> if you have sent some commands using
+I<send_command> method without callback argument and have not yet got all
+replies.
 
 =cut
 
@@ -216,45 +219,43 @@ sub _queue {
     push @{ $self->{_replies} }, $reply;
 }
 
-=head2 $self->send_command($command[, @arguments])
+=head2 $self->send_command($command[, @arguments][, \&callback])
 
-send command to the server. Returns true if command was successfully sent, or
-dies if error occured. Note, that it doesn't return server reply, you should
-retrieve reply using I<get_reply> method.
+send a command to the server. Returns true if the command was successfully
+sent, or dies if an error occured. Note, that it does not return reply from
+the server, you should retrieve it using the I<get_reply> method, or if I<callback>
+has been specified, it will be invoked upon receiving the reply with two
+arguments: the RedisDB object, and the reply from the server.  If the server
+returns an error, the second argument to the callback will be a L<RedisDB::Error>
+object, you can get description of the error using this object in string
+context.  If you are not interested in reply, you can use RedisDB::IGNORE_REPLY
+constant as the last argument.
+
+Note, that RedisDB does not run any background threads, so it will not receive
+the reply and invoke the callback unless you call some of its methods which
+check if there are replies from the server, like I<send_command>,
+I<reply_ready>, I<get_reply>, or I<get_all_replies>.
 
 =cut
 
 sub send_command {
     my $self = shift;
-    ++$self->{_to_be_fetched};
-    return $self->send_command_cb( @_, \&_queue );
-}
 
-sub _ignore { 1 }
+    my $callback;
+    if ( ref $_[-1] eq 'CODE' ) {
+        $callback = pop;
+    }
+    else {
+        ++$self->{_to_be_fetched};
+        $callback = \&_queue;
+    }
 
-=head2 $self->send_command_cb($command[, @arguments][, \&callback])
-
-send command to the server, invoke specified I<callback> on reply. Callback
-invoked with 2 arguments: RedisDB object, and reply from the server.  If server
-return error reply will be L<RedisDB::Error> object, you can get description of
-error using this object in string context.  If I<callback> is not specified
-reply will be discarded.  Note, that RedisDB doesn't run any background
-threads, so it will not receive reply and invoke callback unless you call some
-of it's methods which check if there's reply from the server, like
-I<send_command>, I<send_command_cb>, I<reply_ready>, I<get_reply>, or
-I<get_all_replies>.
-
-=cut
-
-sub send_command_cb {
-    my $self = shift;
-    my $callback = pop if ref $_[-1] eq 'CODE';
-    $callback ||= \&_ignore;
+    my $command = uc shift;
     if ( $self->{_subscription_loop} ) {
         croak "only (UN)(P)SUBSCRIBE and QUIT allowed in subscription loop"
-          unless $_[0] =~ /^(p?(un)?subscribe|quit)$/i;
+          unless $command =~ /^(P?(UN)?SUBSCRIBE|QUIT)$/;
     }
-    my $request = _build_redis_request(@_);
+    my $request = _build_redis_request( $command, @_ );
     $self->_connect unless $self->{_socket} and $self->{_pid} == $$;
 
     # Here we reading received data and storing it in the _buffer,
@@ -267,10 +268,57 @@ sub send_command_cb {
     return 1;
 }
 
+sub _ignore {
+    my ( $self, $res ) = @_;
+    if ( ref $res eq 'RedisDB::Error' ) {
+        warn "Ignoring error returned by redis-server: $res";
+    }
+}
+
+sub IGNORE_REPLY { return \&_ignore; }
+
+=head2 $self->send_command_cb($command[, @arguments][, \&callback])
+
+send a command to the server, invoke specified I<callback> on reply. The
+callback is invoked with two arguments: the RedisDB object, and reply from the
+server.  If the server returned an error, the second argument will be a
+L<RedisDB::Error> object, you can get description of the error using this
+object in string context.  If the I<callback> is not specified, the reply will
+be discarded.  Note, that RedisDB does not run any background threads, so it
+will not receive the reply and invoke the callback unless you call some of its
+methods which check if there are replies from the server, like I<send_command>,
+I<send_command_cb>, I<reply_ready>, I<get_reply>, or I<get_all_replies>.
+
+B<DEPRECATED:> this method is deprecated and may be removed in some future
+version. Please use I<send_command> method instead. If you are using
+I<send_command_cb> with I<&callback> argument, you can just replace the method
+with I<send_command> and it will do the same. If you are using
+I<send_command_cb> with the default callback, you should add the
+RedisDB::IGNORE_REPLY constant as the last argument when replacing the method
+with I<send_command>.  Here is the example that shows equivalents with
+I<send_command>:
+
+    $redis->send_command_cb("SET", "Key", "Value");
+    # may be replaced with
+    $redis->send_command("SET", "Key", "Value", RedisDB::IGNORE_REPLY);
+
+    $redis->send_command_cb("GET", "Key", \&process_reply);
+    # may be replaced with
+    $redis->send_command("GET", "Key", \&process_reply);
+
+=cut
+
+sub send_command_cb {
+    my $self = shift;
+    my $callback = pop if ref $_[-1] eq 'CODE';
+    $callback ||= \&_ignore;
+    return $self->send_command( @_, $callback );
+}
+
 =head2 $self->reply_ready
 
-This method may be used in pipelining mode to check if there are
-some replies already received from server. Returns number of replies
+This method may be used in the pipelining mode to check if there are some
+replies already received from the server. Returns the number of replies
 available for reading.
 
 =cut
@@ -282,9 +330,40 @@ sub reply_ready {
     return @{ $self->{_replies} } ? 1 : 0;
 }
 
+=head2 $self->mainloop
+
+this method blocks till all replies from the server will be received
+
+=cut
+
+sub mainloop {
+    my $self = shift;
+
+    while ( @{ $self->{_callbacks} } ) {
+        die "You can't call mainloop in the child process" unless $self->{_pid} == $$;
+        my $ret = $self->{_socket}->recv( my $buffer, 4096 );
+        unless ( defined $ret ) {
+            next if $! == EINTR;
+            croak "Error reading reply from server: $!";
+        }
+        if ( $buffer ne '' ) {
+
+            # received some data
+            $self->{_buffer} .= $buffer;
+            1 while $self->_parse_reply;
+        }
+        else {
+
+            # disconnected
+            die "Server unexpectedly closed connection before sending full reply";
+        }
+    }
+    return;
+}
+
 =head2 $self->get_reply
 
-receive reply from the server. Method croaks if server returns error reply.
+receive reply from the server. Method croaks if the server returns an error.
 
 =cut
 
@@ -322,7 +401,7 @@ sub get_reply {
 
 =head2 $self->get_all_replies
 
-Wait replies to all sent commands and return them as a list.
+Wait for the replies to all the commands sent and return them as a list.
 
 =cut
 
@@ -337,7 +416,8 @@ sub get_all_replies {
 
 =head2 $self->replies_to_fetch
 
-Return number of commands sent to server replies to which wasn't yet retrieved with I<get_reply> or I<get_all_replies>.
+Return the number of commands sent to the server replies to which wasn't yet
+retrieved with I<get_reply> or I<get_all_replies>.
 
 =cut
 
@@ -348,9 +428,9 @@ sub replies_to_fetch {
 
 =head2 $self->version
 
-Return version of the server client is connected to. Version is returned as floating point
-number represented the same way as the perl versions. E.g. for redis 2.1.12 it will return
-2.001012.
+Return the version of the server client is connected to. The version is
+returned as a floating point number represented the same way as the perl
+versions. E.g. for redis 2.1.12 it will return 2.001012.
 
 =cut
 
@@ -382,27 +462,42 @@ my @commands = qw(
   zscore	zunionstore
 );
 
-=head1 SUPPORTED REDIS COMMANDS
+=head1 WRAPPER METHODS
 
-Usually, instead of using I<execute> method, you can just use methods with names
-matching names of the redis commands. The following methods are defined as wrappers around execute:
-append, auth, bgrewriteaof, bgsave, blpop, brpop, brpoplpush, config_get,
-config_set, config_resetstat, dbsize, debug_object, debug_segfault,
-decr, decrby, del, echo, exists, expire, expireat, flushall,
-flushdb, get, getbit, getrange, getset, hdel, hexists, hget, hgetall,
-hincrby, hkeys, hlen, hmget, hmset, hset, hsetnx, hvals, incr, incrby,
-keys, lastsave, lindex, linsert, llen, lpop, lpush, lpushx,
-lrange, lrem, lset, ltrim, mget, move, mset, msetnx, persist, ping,
-publish, quit, randomkey, rename, renamenx, rpop, rpoplpush,
-rpush, rpushx, sadd, save, scard, sdiff, sdiffstore, select, set,
-setbit, setex, setnx, setrange, sinter, sinterstore,
-sismember, slaveof, smembers, smove, sort, spop, srandmember,
-srem, strlen, sunion, sunionstore, sync, ttl, type, unwatch, watch, zadd, zcard,
-zcount, zincrby, zinterstore, zrange, zrangebyscore, zrank, zrem, zremrangebyrank,
-zremrangebyscore, zrevrange, zrevrangebyscore, zrevrank,
+Instead of using I<execute> and I<send_command> methods directly, it may be
+more convenient to use wrapper methods with names matching names of the redis
+commands. These methods call I<execute> or I<send_command> depending on the
+presence of the callback argument. If callback is specified, the method invokes
+I<send_command> and returns as soon as the command has been sent to the server;
+when the reply is received, it will be passed to the callback (see
+L</"PIPELINING SUPPORT">). If there is no callback, the method invokes
+I<execute>, waits for the reply from the server, and returns it. E.g.:
+
+    $val = $redis->get($key);
+    # equivalent to
+    $val = $redis->execute("get", $key);
+
+    $redis->get($key, sub { $val = $_[1] });
+    # equivalent to
+    $redis->send_command("get", $key, sub { $val = $_[1] });
+
+The following wrapper methods are defined: append, auth, bgrewriteaof, bgsave,
+blpop, brpop, brpoplpush, config_get, config_set, config_resetstat, dbsize,
+debug_object, debug_segfault, decr, decrby, del, echo, exists, expire,
+expireat, flushall, flushdb, get, getbit, getrange, getset, hdel, hexists,
+hget, hgetall, hincrby, hkeys, hlen, hmget, hmset, hset, hsetnx, hvals, incr,
+incrby, keys, lastsave, lindex, linsert, llen, lpop, lpush, lpushx, lrange,
+lrem, lset, ltrim, mget, move, mset, msetnx, persist, ping, publish, quit,
+randomkey, rename, renamenx, rpop, rpoplpush, rpush, rpushx, sadd, save, scard,
+sdiff, sdiffstore, select, set, setbit, setex, setnx, setrange, sinter,
+sinterstore, sismember, slaveof, smembers, smove, sort, spop, srandmember,
+srem, strlen, sunion, sunionstore, sync, ttl, type, unwatch, watch, zadd,
+zcard, zcount, zincrby, zinterstore, zrange, zrangebyscore, zrank, zrem,
+zremrangebyrank, zremrangebyscore, zrevrange, zrevrangebyscore, zrevrank,
 zscore, zunionstore
 
-See description of all commands in redis documentation at L<http://redis.io/commands>.
+See description of all commands in redis documentation at
+L<http://redis.io/commands>.
 
 =cut
 
@@ -412,7 +507,12 @@ for my $command (@commands) {
     no strict 'refs';
     *{ __PACKAGE__ . "::$command" } = sub {
         my $self = shift;
-        return $self->execute( $uccom, @_ );
+        if ( ref $_[-1] eq 'CODE' ) {
+            return $self->send_command( $uccom, @_ );
+        }
+        else {
+            return $self->execute( $uccom, @_ );
+        }
     };
 }
 
@@ -424,8 +524,9 @@ The following commands implement some additional postprocessing of results:
 
 =head2 $self->info
 
-Return information and statistics about server. Redis returns information in form of
-I<field:value>, I<info> method parses result and returns it as hash reference.
+return information and statistics about the server. Redis-server returns
+information in form of I<field:value>, the I<info> method parses result and
+returns it as a hash reference.
 
 =cut
 
@@ -439,8 +540,8 @@ sub info {
 
 =head2 $self->shutdown
 
-Shuts redis server down. Returns undef, as server doesn't send answer.
-Croaks in case of error.
+Shuts the redis server down. Returns undef, as the server doesn't send the
+answer.  Croaks in case of the error.
 
 =cut
 
@@ -453,52 +554,89 @@ sub shutdown {
 
 =head1 HANDLING OF SERVER DISCONNECTS
 
-Redis server may close connection if it was idle for some time, also connection
-may be closed in case redis-server was restarted. RedisDB restores connection
-to the server but only if no data was lost as result of disconnect. E.g. if
-client was idle for some time and redis server closed connection, it will be
-transparently restored on sending next command. If you send a command and
-server closed connection without sending complete reply, connection will not be
-restored and module will throw exception. Also module will throw exception if
-connection will be closed in the middle of transaction or while you're in
+Redis server may close a connection if it was idle for some time, also the
+connection may be closed in case when redis-server was restarted. RedisDB
+restores a connection to the server but only if no data was lost as a result of
+disconnect. E.g. if the client was idle for some time and the redis server
+closed the connection, it will be transparently restored on sending next
+command. If you have sent a command and the server has closed the connection
+without sending complete reply, the connection will not be restored and the
+module will throw an exception. Also the module will throw an exception if the
+connection was closed in the middle of a transaction or while you're in a
 subscription loop.
 
 =cut
 
 =head1 PIPELINING SUPPORT
 
-You can send commands in pipelining mode. In this case you sending multiple
-commands to the server without waiting for replies.  You can use
-I<send_command> and I<send_command_cb> methods to send multiple commands to the
-server.  In case of I<send_command> I<reply_ready> method may be used to check
-if some replies are already received, and I<get_reply> method may be used to
-fetch received reply. In case of I<send_command_cb> redis will invoke specified
-callback when it receive reply. Also if you not interested in reply you can
-omit callback, then reply will be dropped. Note, that RedisDB checks for
-replies only when you calling some of its methods, so the following loop will
-never exit:
+You can send commands in the pipelining mode. It means you are sending multiple
+commands to the server without waiting for the replies.  This is implemented by
+the I<send_command> method. Recommended way of using it is to pass a reference
+to the callback function as the last argument.  When module receives reply from
+the server, it will call this function with two arguments: reference to the
+RedisDB object, and reply from the server. It is important to understand
+though, that RedisDB does not run any background threads, neither it checks for
+the replies by setting some timer, so e.g. in the following example callback
+will never be invoked:
 
-    my $reply;
-    $redis->send_command_cb("PING", sub { $reply = $_[1] });
-    sleep 1 while not $reply;
+    my $pong;
+    $redis->send_command( "ping", sub { $pong = $_[1] } );
+    sleep 1 while not $pong;    # this will never return
 
-because redis will never receive reply and invoke callback.
+Therefore you need perriodically trigger check for the replies. The check is
+triggered when you call the following methods: I<send_command>, I<reply_ready>,
+I<get_reply>, I<get_all_replies>.  Calling wrapper method, like
+C<< $redis->get('key') >>, will also trigger check as internally wrapper methods
+use methods listed above.
 
-Note also, that you can't use I<execute> method (or wrappers around it, like
-I<get> or I<set>) while in pipeline mode, you must receive replies on all
-pipelined commands first.
+If you invoke I<send_command> without a callback argument, you have to fetch
+reply later explicitly using I<get_reply> method. This is how synchronous
+I<execute> is implemented, basically it is:
+
+    sub execute {
+        my $self = shift;
+        $self->send_command(@_);
+        return $self->get_reply;
+    }
+
+That is why it is not allowed to call I<execute> if you have not got replies to
+all commands sent previously with I<send_command> without callback.
+
+Sometimes you are not interested in replies sent by the server, e.g. SET
+command usually just return 'OK', in this case you can pass to I<send_command>
+callback which ignores its arguments, or use C<RedisDB::IGNORE_REPLY> constant, it
+is a no-op function:
+
+    for (@keys) {
+        # execute will not just send 'GET' command to the server,
+        # but it will also receive response to the 'SET' command sent on
+        # the previous loop iteration
+        my $val = $redis->execute( "get", $_ );
+        $redis->send_command( "set", $_, fun($val), RedisDB::IGNORE_REPLY );
+    }
+    # and this will wait for the last reply
+    $redis->mainloop;
+
+or using L</"WRAPPER METHODS"> you can rewrite it as:
+
+    for (@keys) {
+        my $val = $redis->get($_);
+        $redis->set( $_, fun($val), RedisDB::IGNORE_REPLY );
+    }
+    $redis->mainloop;
 
 =cut
 
 =head1 SUBSCRIPTIONS SUPPORT
 
-RedisDB supports subscriptions to redis channels. In subscription mode you can
-subscribe to some channels and receive all messages sent to these channels.
-Every time RedisDB receives message for the channel it invokes callback
-provided by user. User can specify different callbacks for different channels.
-When in subscription mode you can subscribe to additional channels, or
-unsubscribe from channels you subscribed to, but you can't use any other redis
-commands like set, get, etc. Here's example of running in subscription mode:
+RedisDB supports subscriptions to redis channels. In the subscription mode you
+can subscribe to some channels and receive all the messages sent to these
+channels.  Every time RedisDB receives a message for the channel it invokes a
+callback provided by the user. User can specify different callbacks for the
+different channels.  When in the subscription mode you can subscribe to
+additional channels, or unsubscribe from the channels you subscribed to, but
+you can't use any other redis commands like set, get, etc. Here is the example
+of running in the subscription mode:
 
     my $message_cb = sub {
         my ($redis, $channel, $pattern, $message) = @_;
@@ -522,17 +660,18 @@ commands like set, get, etc. Here's example of running in subscription mode:
         default_callback => $message_cb,
     );
 
-subscription_loop will subscribe you to news channel and control.* channels. It
-will call specified callbacks every time new message received.  You can
-subscribe to additional channels sending their names to control.subscribe
-channel. You can unsubscribe from all channels by sending message to
-control.quit channel. Every callback receives four arguments: RedisDB object,
-channel for which message was received, pattern if you subscribed to this
-channel using I<psubscribe> method, and message itself.
+subscription_loop will subscribe you to the news channel and control.*
+channels. It will call specified callbacks every time a new message received.
+You can subscribe to additional channels sending their names to the
+control.subscribe channel. You can unsubscribe from all the channels by sending
+a message to the control.quit channel. Every callback receives four arguments:
+the RedisDB object, the channel for which the message was received, the pattern
+if you subscribed to this channel using I<psubscribe> method, and the message
+itself.
 
-You can publish messages into channels using I<publish> method. This method
-should be called when you in normal mode, and can't be used while you're in
-subscription mode.
+You can publish messages into the channels using the I<publish> method. This
+method should be called when you in the normal mode, and can't be used while
+you're in the subscription mode.
 
 Following methods can be used in subscribtion mode:
 
@@ -540,27 +679,28 @@ Following methods can be used in subscribtion mode:
 
 =head2 $self->subscription_loop(%parameters)
 
-Enter into subscription mode. Function subscribes you to specified channels,
-waits for messages, and invokes callbacks for every received message. Function
-returns after you unsubscribed from all channels. It accepts following parameters:
+Enter into the subscription mode. The method subscribes you to the specified
+channels, waits for the messages, and invokes the appropriate callback for
+every received message. The method returns after you unsubscribed from all the
+channels. It accepts the following parameters:
 
 =over 4
 
 =item default_callback
 
-reference to the default callback. This callback is invoked for the message if you
+reference to the default callback. This callback is invoked for a message if you
 didn't specify other callback for the channel this message comes from.
 
 =item subscribe
 
-array reference. Contains list of channels you want to subscribe. Channel name
-may be optionally followed by reference to callback function for this channel.
-E.g.:
+an array reference. Contains the list of channels you want to subscribe. A
+channel name may be optionally followed by the reference to a callback function
+for this channel.  E.g.:
 
     [ 'news', 'messages', 'errors' => \&error_cb, 'other' ]
 
 channels "news", "messages", and "other" will use default callback, but for
-"errors" channel error_cb function will be used.
+the "errors" channel error_cb function will be used.
 
 =item psubscribe
 
@@ -569,8 +709,8 @@ same as subscribe, but you specify patterns for channels' names.
 =back
 
 All parameters are optional, but you must subscribe at least to one channel. Also
-if default_callback is not specified, you have to explicitely specify callback
-for every channel you're going to subscribe.
+if default_callback is not specified, you have to explicitely specify a callback
+for every channel you are going to subscribe.
 
 =cut
 
@@ -635,8 +775,8 @@ sub subscription_loop {
 
 =head2 $self->subscribe($channel[, $callback])
 
-Subscribe to additional I<$channel>. If I<$callback> is not specified, default
-callback will be used.
+Subscribe to the additional I<$channel>. If I<$callback> is not specified,
+default callback will be used.
 
 =cut
 
@@ -671,8 +811,8 @@ sub psubscribe {
 
 =head2 $self->unsubscribe([@channels])
 
-Unsubscribe from the listed I<@channels>. If no channels specified, unsubscribe
-from all channels.
+Unsubscribe from the listed I<@channels>. If no channels was specified,
+unsubscribe from all the channels.
 
 =cut
 
@@ -683,8 +823,8 @@ sub unsubscribe {
 
 =head2 $self->punsubscribe([@patterns])
 
-Unsubscribe from the listed I<@patterns>. If no patterns specified, unsubscribe
-from all channels to which you subscribed using I<psubscribe>.
+Unsubscribe from the listed I<@patterns>. If no patterns was specified,
+unsubscribe from all the channels to which you subscribed using I<psubscribe>.
 
 =cut
 
@@ -715,26 +855,26 @@ sub psubscribed {
 
 =head1 TRANSACTIONS SUPPORT
 
-Transactions allow you execute a sequence of commands in a single step. In
-order to start transaction you should use method I<multi>.  After you entered
-transaction all commands you issue are queued, but not executed till you call
-I<exec> method. Tipically these commands return string "QUEUED" as result, but
-if there's an error in e.g. number of arguments they may croak. When you
-calling exec all queued commands are executed and exec returns list of results
-for every command in transaction. If any command failed exec will croak. If
-instead of I<exec> you will call I<discard>, all scheduled commands will be
-canceled.
+Transactions allow you to execute a sequence of commands in a single step. In
+order to start a transaction you should use the I<multi> method.  After you
+have entered a transaction all the commands you issue are queued, but not
+executed till you call the I<exec> method. Tipically these commands return
+string "QUEUED" as result, but if there is an error in e.g. number of
+arguments, they may croak. When you call exec, all the queued commands will be
+executed and exec will return a list of results for every command in the
+transaction. If any command has failed, exec will croak. If instead of I<exec>
+you call I<discard>, all scheduled commands will be canceled.
 
-You can set some keys as watched. If any whatched key will be changed by
-another client before you call exec, transaction will be discarded and exec
-will return false value.
+You can set some keys as watched. If any whatched key has been changed by
+another client before you called exec, the transaction will be discarded and
+exec will return false value.
 
 =cut
 
 =head2 $self->multi
 
-Enter transaction. After this and till I<exec> or I<discard> will be called,
-all commands will be queued but not executed.
+Enter the transaction. After this and till I<exec> or I<discard> will be called,
+all the commands will be queued but not executed.
 
 =cut
 
@@ -748,10 +888,10 @@ sub multi {
 
 =head2 $self->exec
 
-Execute all queued commands and finish transaction. Returns list of results for
-every command. May croak if some command failed.  Also unwatches all keys. If
-some of the watched keys was changed by other client, transaction will be
-canceled and I<exec> will return false.
+Execute all queued commands and finish the transaction. Returns a list of
+results for every command. Will croak if some command has failed.  Also
+unwatches all the keys. If some of the watched keys has been changed by other
+client, the transaction will be canceled and I<exec> will return false.
 
 =cut
 
@@ -1012,15 +1152,14 @@ L<Redis>, L<Redis::hiredis>, L<AnyEvent::Redis>
 
 =head1 WHY ANOTHER ONE
 
-I was in need of a client for redis database. L<AnyEvent::Redis> didn't suite
-me as it requires event loop, and it didn't fit into existing code. Problem
-with L<Redis> is that it doesn't (at the time I write this) reconnect to the
-server if connection was closed after timeout or as result or server restart,
-and it doesn't support pipelining. After analizing what I need to change in
-L<Redis> in order to get all I want (see TODO), I decided that it will be
-simplier to write new module from scratch. This also solves the problem of
-backward compatibility. Pedro Melo, maintainer of L<Redis> have plans to
-implement some of these features too.
+I was in need of the client for redis database. L<AnyEvent::Redis> didn't suite
+me as it requires an event loop, and it didn't fit into the existing code. The
+problem with L<Redis> is that it didn't (at the time I started this) reconnect
+to the server if connection was closed after timeout or as result of the server
+restart, and it does not support pipelining. After analizing what I need to
+change in L<Redis> in order to get all I want, I decided that it will be
+simplier to write the new module from scratch. This also solves the problem of
+backward compatibility.
 
 =head1 BUGS
 
